﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Collections.Specialized;
using System.Text;
using System.Collections.ObjectModel;

using IdeaBlade.Core;
using IdeaBlade.EntityModel;
using IdeaBlade.Validation;
using System.ComponentModel;

namespace WpfSimpleSteps {
  class MainWindowViewModel : INotifyPropertyChanged {
    public ObservableCollection<Employee> Employees { get; private set; }
    public ObservableCollection<string> Log { get; private set; }

    public MainWindowViewModel() {
      ASyncMainWindowViewModel();
    }

    public async void ASyncMainWindowViewModel() {
      Log = new ObservableCollection<string>();
      Employees = new ObservableCollection<Employee>();
      Employees.CollectionChanged += Employees_CollectionChanged;

      _mgr = new NorthwindIBEntities();
      _mgr.EntityChanged += (s, args) => HasChanges = ((NorthwindIBEntities)s).HasChanges();
      _mgr.Fetching += new EventHandler<EntityFetchingEventArgs>(_mgr_Fetching);

      //_query = _mgr.Employees.Include("Orders");
      _query = _mgr.Employees;
      (await _query.ExecuteAsync()).ForEach(Employees.Add);

      WriteToLog("Initializing View Model");

      //_mgr.VerifierEngine.DefaultVerifierOptions.ErrorNotificationMode = VerifierErrorNotificationMode.NotifyAndThrowException;
    }

    void _mgr_Fetching(object sender, EntityFetchingEventArgs e) {
      WriteToLog("Fetching " + e.Query.ElementType);
    }

    public Employee CurrentEmployee {
      get { return _currentEmployee; }
      private set {
        _currentEmployee = value;
        RaisePropertyChanged("CurrentEmployee");
      }
    }

    public int CurrentPosition {
      get { return _currentPosition; }
      set {
        if (Employees != null && Employees.Count > 0) {
          _currentPosition = Math.Min(Math.Max(value, 0), Employees.Count - 1);
          CurrentEmployee = Employees[_currentPosition];
        } else {
          _currentPosition = -1;
          CurrentEmployee = null;
        }
        RaisePropertyChanged("CurrentPosition");
      }
    }

    public bool HasChanges {
      get { return _hasChanges; }
      private set {
        _hasChanges = value;
        RaisePropertyChanged("HasChanges");
      }
    }

    public void MoveCurrentToFirst() {
      CurrentPosition = 0;
    }

    public void MoveCurrentToPrevious() {
      CurrentPosition = CurrentPosition - 1;
    }

    public void MoveCurrentToNext() {
      CurrentPosition = CurrentPosition + 1;
    }

    public void MoveCurrentToLast() {
      CurrentPosition = Employees != null ? Employees.Count - 1 : -1;
    }

    public void AddNew() {
      Employees.Add(new Employee());
      MoveCurrentToLast();
    }

    public void RemoveAt(int position) {
      Employees.RemoveAt(position);
      CurrentPosition = position;
    }

    public void Save() {
      _mgr.SaveChanges();
    }

    public void RollbackChanges() {
      int currentPosition = CurrentPosition;

      _mgr.RejectChanges();
      Employees.Clear();
      _query.With(QueryStrategy.CacheOnly).Execute().ForEach(Employees.Add);

      CurrentPosition = currentPosition;
    }

    private void RaisePropertyChanged(string property) {
      PropertyChanged(this, new PropertyChangedEventArgs(property));
    }

    private void Employees_CollectionChanged(object sender, NotifyCollectionChangedEventArgs e) {
      switch (e.Action) {
        case NotifyCollectionChangedAction.Add:
          Add(e.NewItems.Cast<Employee>());
          break;
        case NotifyCollectionChangedAction.Remove:
          Delete(e.OldItems.Cast<Employee>());
          break;
      }
    }

    private void Add(IEnumerable<Employee> employees) {
      employees.ForEach(_mgr.AddEntity);
    }

    private void Delete(IEnumerable<Employee> employees) {
      employees.ForEach(employee => employee.EntityAspect.Delete());
      Save();
    }

    private void WriteToLog(string message) {
      Log.Insert(0, message);
    }

    public event PropertyChangedEventHandler PropertyChanged = delegate { };

    private Employee _currentEmployee = null;
    private int _currentPosition = -1;
    private NorthwindIBEntities _mgr;
    private bool _hasChanges = false;
    private EntityQuery<Employee> _query;
  }
}
